#pragma once

#include <mutex>
#include <condition_variable>
#include <deque>

template <typename T>

/* Definice tridy implementujici blokujici frontu. */
class BlockingQueue
{

    private:

        /* Mutex na uzamceni kriticke sekce. */
        std::mutex d_mutex;

        /* Podminkova promenna pro notifikace. */
        std::condition_variable d_condition;

        /* Fronta pro ukladani obsahu. */
        std::deque<T> d_queue;

    public:

        /* Metoda pro pridani prvku do fronty. */
        void push(T const& value)
        {
            {
                std::unique_lock<std::mutex> lock(this->d_mutex);
                d_queue.push_front(value);
            }
            this->d_condition.notify_one();
        }

        /* Metoda pro odebrani prvku z fronty. */
        T pop()
        {
            std::unique_lock<std::mutex> lock(this->d_mutex);
            this->d_condition.wait(lock, [=] { return !this->d_queue.empty(); });
            T rc(std::move(this->d_queue.back()));
            this->d_queue.pop_back();
            return rc;
        }

        /* Metoda pro zjisteni, jestli je fronta prazdna. */
        bool empty()
        {
            std::unique_lock<std::mutex> lock(this->d_mutex);
            return d_queue.empty();
        }
};