#include "Day.h"

Day::Day(std::string newNameOfDay, int weekNumber) : QWidget(nullptr)
{
	// Nastaveni nazvu dne.
	this->nameOfDay = newNameOfDay;

	dayFrame = new QFrame();

	// Naplneni vektoru potrebnymi hodnotami.
	dayEvents.push_back(new DayEventValues(BREAKFAST_TIME_HOURS));
	dayEvents.push_back(new DayEventValues(SNACK_MORNING_TIME_HOURS));
	dayEvents.push_back(new DayEventValues(LUNCH_TIME_HOURS));
	dayEvents.push_back(new DayEventValues(SNACK_AFTERNOON_HOURS));
	dayEvents.push_back(new DayEventValues(DINNER_TIME_HOURS));
	dayEvents.push_back(new DayEventValues(MOVEMENT_TIME_HOURS));

	eventsWidgets.push_back(new DayEventWidgets("Breakfast"));
	eventsWidgets.push_back(new DayEventWidgets("Morning snack"));
	eventsWidgets.push_back(new DayEventWidgets("Lunch"));
	eventsWidgets.push_back(new DayEventWidgets("Afternoon snack"));
	eventsWidgets.push_back(new DayEventWidgets("Dinner"));
	eventsWidgets.push_back(new DayEventWidgets("Movement"));

	centralBarLayout = new QGridLayout();
	weekLabel = new QLabel("Week: ");
	numOfWeekLabel = new QLabel(QString::number(weekNumber));
	dayName = new QLabel(QString::fromStdString(nameOfDay));
	activityLabel = new QLabel("Movement");
	foodLabel = new QLabel("Sugar (g)");

	int x = 0;

	centralBarLayout->addWidget(dayName, x, 0, 1, 2, Qt::AlignCenter);
	centralBarLayout->addWidget(weekLabel, ++x, 0, 1, 1, Qt::AlignRight);
	centralBarLayout->addWidget(numOfWeekLabel, x, 1, 1, 1, Qt::AlignLeft);

	centralBarLayout->addWidget(foodLabel, ++x, 0, 1, 2, Qt::AlignCenter);

	for (int i = 0; i < eventsWidgets.size(); ++i) {
		if (i == eventsWidgets.size() - 1) {
			centralBarLayout->addWidget(activityLabel, ++x, 0, 1, -1, Qt::AlignCenter);
		}
		eventsWidgets[i]->getEventCheck()->setChecked(dayEvents[i]->getEventIsChecked());
		centralBarLayout->addWidget(eventsWidgets[i]->getEventCheck(), ++x, 0, 1, 1);
		eventsWidgets[i]->getEventBox()->setValue(dayEvents[i]->getEventValue());
		centralBarLayout->addWidget(eventsWidgets[i]->getEventBox(), x, 1, 1, 2);
		eventsWidgets[i]->getEventTime()->setTime(QTime(dayEvents[i]->getEventTimeHours(), dayEvents[i]->getEventTimeMinutes()));
		centralBarLayout->addWidget(eventsWidgets[i]->getEventTime(), ++x, 0, 1, 1, Qt::AlignLeft);
		centralBarLayout->addWidget(eventsWidgets[i]->getEventJitterLabelZero(), x, 0, 1, 1, Qt::AlignRight);
		centralBarLayout->addWidget(eventsWidgets[i]->getEventJitterLabelOne(), x, 2, 1, 1, Qt::AlignLeft);
		eventsWidgets[i]->getEventJitterSlider()->setValue(dayEvents[i]->getEventJitter());
		centralBarLayout->addWidget(eventsWidgets[i]->getEventJitterSlider(), x, 1, 1, 1, Qt::AlignCenter);
	}

	dayFrame->setLayout(centralBarLayout);
	dayFrame->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	for (int i = 0; i < eventsWidgets.size(); ++i) {

		// Checkbox pro urceni pouzitiy aktivity.
		connect(eventsWidgets[i]->getEventCheck(), &QCheckBox::stateChanged,
			[=](const bool) {
				this->saveDay();
			});

		// DoubleSpinbox pro urceni hodnoty aktivity.
		connect(eventsWidgets[i]->getEventBox(), QOverload<double>::of(&QDoubleSpinBox::valueChanged),
			[=](int index) {
				this->saveDay();
			});

		// QTime pro urceni casu - ukladaji se zvlast hodiny a minuty.
		connect(eventsWidgets[i]->getEventTime(), &QTimeEdit::timeChanged,
			[=](const QTime& t) {
				this->saveDay();
			});

		// Slider pro urceni nahodnosti.
		connect(eventsWidgets[i]->getEventJitterSlider(), &QSlider::valueChanged,
			[=](int index) {
				this->saveDay();
			});
	}
}

Day::~Day()
{
	// Vycisteni vektoru.
	dayEvents.clear();
}

void Day::saveDay()
{
	for (int i = 0; i < eventsWidgets.size(); ++i) {
		// Je potreba ulozit vsechny udalosti dne.
		dayEvents[i]->saveEvent(eventsWidgets[i]);
	}
}

std::string Day::getNameOfDay()
{
	return this->nameOfDay;
}

QFrame* Day::getDayFrame()
{
	return this->dayFrame;
}

std::vector<DayEventValues*> Day::getDayEvents()
{
	return this->dayEvents;
}

DayEventValues * Day::getDayEvent(int index)
{
	return dayEvents[index];
}
