#include "ExecutionWindow.h"

ExecutionWindow::ExecutionWindow()
{
	// Nastaveni nazvu okna.
	setWindowTitle("In-silico_evaluation");

	// Inicializace promennych.
	configsDelivered = 0;
	firstXValue = 0;
	donePercentage = 0;
	numOfMetrics = 0;
	chosenIndex = 0;

	// Inicializace widgetu.
	centralWidget = new QWidget();
	centralLayout = new QGridLayout();

	configsDirLabel = new QLabel("Configs directory:");
	configsDir = new QLineEdit("patients_configs");

	scenarioFileLabel = new QLabel("Scenario file:");
	scenarioFileEdit = new QLineEdit("generated_scenario.csv");

	outputFileLabel = new QLabel("Output file:");
	outputFile = new QLineEdit("output_file.csv");

	metricsLabel = new QLabel("Metric: ");
	metricsBox = new QComboBox();

	checkAll = new QLabel("Count all metrics:");

	checkAllBox = new QCheckBox();
	checkAllBox->setChecked(false);

	firstXLabel = new QLabel("Number of patients: ");;

	firstXBox = new QSpinBox();
	firstXBox->setFixedWidth(60);
	firstXBox->setMaximum(MAX_X_FILES);

	statusLabel = new QLabel("Status");
	progress = new QProgressBar();
	progress->setTextVisible(true);
	startButton = new QPushButton("Start");

	solversLabel = new QLabel("Solver: ");
	solversBox = new QComboBox();

	populationLabel = new QLabel("Population: ");
	populationBox = new QSpinBox();
	populationBox->setMinimum(population);
	populationBox->setMaximum(INT_MAX);

	generationLabel = new QLabel("Generation: ");
	generationBox = new QSpinBox();
	generationBox->setMinimum(generation);
	generationBox->setMaximum(INT_MAX);

	// Ziskani vsech metric dostupnych ve SMartCGMS.
	foreach(scgms::TMetric_Descriptor metric, scgms::get_metric_descriptors()) {
		metrics.push_back(metric);
		metricsBox->addItem(QString::fromStdWString(metric.description));
	}

	// Ziskani solveru dostupnych ve SmartCGMS.
	foreach(scgms::TSolver_Descriptor solver, scgms::get_solver_descriptors()) {
		solvers.push_back(solver);
		solversBox->addItem(QString::fromStdWString(solver.description));
	}

	// Rozlozeni widgetu.
	int x = 0;

	centralLayout->addWidget(configsDirLabel, ++x, 0, 1, 1, Qt::AlignRight);
	centralLayout->addWidget(configsDir, x, 1, 1, 1);

	centralLayout->addWidget(scenarioFileLabel, ++x, 0, 1, 1, Qt::AlignRight);
	centralLayout->addWidget(scenarioFileEdit, x, 1, 1, 1);

	centralLayout->addWidget(outputFileLabel, ++x, 0, 1, 1, Qt::AlignRight);
	centralLayout->addWidget(outputFile, x, 1, 1, 1);

	centralLayout->addWidget(solversLabel, ++x, 0, 1, 1, Qt::AlignRight);
	centralLayout->addWidget(solversBox, x, 1, 1, 1);

	centralLayout->addWidget(metricsLabel, ++x, 0, 1, 1, Qt::AlignRight);
	centralLayout->addWidget(metricsBox, x, 1, 1, 1);

	centralLayout->addWidget(checkAll, ++x, 0, 1, 1, Qt::AlignRight);
	centralLayout->addWidget(checkAllBox, x, 1, 1, 1, Qt::AlignLeft);

	centralLayout->addWidget(firstXLabel, ++x, 0, 1, 1, Qt::AlignRight);
	centralLayout->addWidget(firstXBox, x, 1, 1, 1, Qt::AlignLeft);

	centralLayout->addWidget(populationLabel, ++x, 0, 1, 1, Qt::AlignRight);
	centralLayout->addWidget(populationBox, x, 1, 1, 1, Qt::AlignLeft);

	centralLayout->addWidget(generationLabel, ++x, 0, 1, 1, Qt::AlignRight);
	centralLayout->addWidget(generationBox, x, 1, 1, 1, Qt::AlignLeft);

	centralLayout->addWidget(statusLabel, ++x, 0, 1, -1, Qt::AlignCenter);

	progress->setTextVisible(true);
	centralLayout->addWidget(progress, ++x, 0, 1, 2);

	centralLayout->addWidget(startButton, ++x, 0, 1, -1, Qt::AlignCenter);

	centralWidget->setLayout(centralLayout);

	setCentralWidget(centralWidget);

	// 

	// Pripojeni signalu.

	// Tlacitko pro spusteni simulace.
	connect(startButton, SIGNAL(clicked()), this, SLOT(OnStartButtonClick()));

	// Checkbox pro vyber vsech metrik.
	connect(checkAllBox, &QCheckBox::stateChanged,
		[=](const bool) {	chooseAll = checkAllBox->checkState();
		});

	// Combobox pro vyber metriky.
	connect(metricsBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
		[=](int index) {
			if (chooseAll == true) {
				chosenIndex = 0;
			}
			else {
				chosenIndex = metricsBox->currentIndex();
			}});

	// Combobox pro vyber solveru.
	connect(solversBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
		[=](int index) {
			chosen_solver = index;
			});

	// Spinbox pro vyber prvnich x pacientu.
	connect(firstXBox, QOverload<int>::of(&QSpinBox::valueChanged),
		[=](int index) {
			firstXValue = firstXBox->value();
		});

	// Spinbox pro vyber velikosti populace.
	connect(populationBox, QOverload<int>::of(&QSpinBox::valueChanged),
		[=](int index) {
			population = populationBox->value();
		});

	// Spinbox pro vyber poctu generaci.
	connect(generationBox, QOverload<int>::of(&QSpinBox::valueChanged),
		[=](int index) {
			generation = generationBox->value();
		});
}

ExecutionWindow::~ExecutionWindow()
{
	// Vsechna vlakna je potreba dat do join stavu.
	for (int i = 0; i < mSmartCGMSExecutionThreads.size(); ++i) {
		if (mSmartCGMSExecutionThreads[i] && mSmartCGMSExecutionThreads[i]->joinable()) {
			mSmartCGMSExecutionThreads[i]->join();
		}
	}

	// Resetovani pointeru.
	for (int i = 0; i < mSmartCGMSExecutionThreads.size(); ++i) {
		mSmartCGMSExecutionThreads[i].reset();
	}
}

void ExecutionWindow::InvokeShowProgress(int percentage)
{
	// Nastaveni nove hodnoty progressbaru.
	progress->setValue(percentage + donePercentage);

	// Pokud prisel signal, ze byla dopocitana metrika, prida se cela hodnota (100%).
	if (percentage == COUNT_PART_DONE) {
		donePercentage += COUNT_PART_DONE;
	}

	// Vypocet metrik skoncil.
	if (donePercentage == COUNT_PART_DONE * numOfMetrics * configsDelivered) {
		// Povoleni zacatku optimalizace.
		metricsDone = true;
	}

	// Pokud je vse hotovo.
	else if (donePercentage == COUNT_PART_DONE * numOfMetrics * configsDelivered * 2) {

		// Promenna pro ukonceni vlaken.
		running = false;

		// Zpet se uvolni tlacitko start.
		startButton->setDisabled(false);
	}
}

void ExecutionWindow::OnStartButtonClick()
{
	running = true;
	metricsDone = false;
	sortNeeded = true;
	donePercentage = 0;

	// Zablokuje se tlacitko, aby se nemohla spustit dalsi simulace.
	startButton->setDisabled(true);

	// Nastavi se label, ze se pocita.
	statusLabel->setText("Execution in progress!");

	// Promenna slozky, ze ktere se budou cist konfigurace pacientu.
	QDir dir(StdWStringToQString(Get_Application_Dir() / QStringToStdWString(configsDir->text())));

	// Slozka neexistuje.
	if (!dir.exists()) {
		// Vypise se error.
		QMessageBox box(QMessageBox::Icon::Critical, "Error", "The directory does not exists!");
		box.exec();
		return;
	}

	// Vyfiltruje se pouze soubory s .ini priponou.
	dir.setNameFilters(QStringList() << "*.ini");
	// Soubory se seradi podle abecedniho poradi.
	dir.setSorting(QDir::Name);

	// Vytvori se list vyfiltrovanych souboru ve slozce.
	QStringList list = dir.entryList();
	configsDelivered = list.size();

	// Pokud byl vybran pocet pacientu, kteri se maji vybrat.
	if (firstXValue > 0)
		configsDelivered = firstXValue;

	numOfMetrics = metrics.size();

	// Pokud nebyly vybrany vsechny metriky.
	if (chooseAll == false)
		numOfMetrics = 1;

	// Nastaveni 100% hodnoty progressbaru. 
	progress->setMaximum(COUNT_PART_DONE * numOfMetrics * configsDelivered * 2);

	// Pro vsechny nactene konfigurace pacientu se zavola metoda, ktera zavola 
	//vypocet nad poctem vybranych metrik nad jednou konfiguraci.
	for (int i = 0; i < configsDelivered; ++i) {
 		HandleConfig(i * numOfMetrics * 2, QStringToStdWString(list[i]));
	}

	// Docasna promenna pro vytvoreni vlaken.
	std::unique_ptr<std::thread> thread;

	// Vytvori se pocet vlaken, ktera jsou na stroji dostupna.
	for (int i = 0; i < std::thread::hardware_concurrency() - 1; ++i) {
		thread = std::make_unique<std::thread>(&ExecutionWindow::MetricThread, this);
		// Vlakno se prida do vektoru vlaken.
		mSmartCGMSExecutionThreads.push_back(std::move(thread));
	}

	// Navic se vytvori jedno pro optimalizaci.
	thread = std::make_unique<std::thread>(&ExecutionWindow::OptimizeThread, this);
	// Vlakno se prida do vektoru vlaken.
	mSmartCGMSExecutionThreads.push_back(std::move(thread));
}

// Spojka pro hledani filtru - jen zavola metodu okna.
HRESULT IfaceCalling On_Filter_Created_Bridge(scgms::IFilter* filter, const void* data)
{
	ExecutionWindow* w = const_cast<ExecutionWindow*>(reinterpret_cast<const ExecutionWindow*>(data));

	return S_OK;
}

void ExecutionWindow::HandleConfig(int index, const std::wstring& fileName)
{
	// Cesta k souboru konfigurace pacienta.
	const std::wstring configFilename = Get_Application_Dir() / (QStringToStdWString(configsDir->text()) + (std::wstring(1, '/') + fileName));

	int i = 0;

	// Pokud jsou zvoleny vsechny metriky.
	if (chooseAll == true) {
		// Pro vsechny metriky.
		foreach(scgms::TMetric_Descriptor curr_metric, metrics) {
			// Nacte se konfigurace.
			scgms::SPersistent_Filter_Chain_Configuration configuration = LoadConfiguration(configFilename);
			// Nepovedlo se nacist konfiguraci.
			if (!configuration) {
				return;
			}

			// Nastavi se dana metrika.
			SetMetricId(configuration, curr_metric.id);

			// Konfgurace spolu s dalsimy parametry se prida do fronty pro vypocet metrik.
			metricQueue.push(new ThreadObject(++index, configuration, new scgms::TMetric_Descriptor(curr_metric)));
			// Konfgurace spolu s dalsimy parametry se prida do fronty pro vypocet oprimalizace parametru.
			optimizeQueue.push(new ThreadObject(++index, configuration, fileName, new scgms::TMetric_Descriptor(curr_metric)));
		}
	}
	// Byla vybrana pouze jedna metrika.
	else {
		// Nacte se konfigurace.
		scgms::SPersistent_Filter_Chain_Configuration configuration = LoadConfiguration(configFilename);
		// Nepovedlo se nacist konfiguraci.
		if (!configuration) {
			return;
		}

		// Nastavi se dana metrika.
		SetMetricId(configuration, metrics[chosenIndex].id);
		// Konfgurace spolu s dalsimy parametry se prida do fronty pro vypocet metrik.
		metricQueue.push(new ThreadObject(++index, configuration, new scgms::TMetric_Descriptor(metrics[chosenIndex])));
		// Konfgurace spolu s dalsimy parametry se prida do fronty pro vypocet oprimalizace parametru.
		optimizeQueue.push(new ThreadObject(++index, configuration, fileName, new scgms::TMetric_Descriptor(metrics[chosenIndex])));
	}
}

scgms::SPersistent_Filter_Chain_Configuration ExecutionWindow::LoadConfiguration(const std::wstring& config_file)
{

	scgms::SPersistent_Filter_Chain_Configuration configuration;
	refcnt::Swstr_list errors;

	/* Cesta ke config souboru. */
	const std::wstring Config_Path = Get_Application_Dir() / config_file;

	HRESULT rc = E_FAIL;

	// Pokud je konfigurace prazdna.
	if (configuration) {
		rc = configuration->Load_From_File(Config_Path.c_str(), errors.get());
	}

	/* Doslo k nejake chybe. */
	if (!Succeeded(rc))
	{
		// Vypis vsech erroru.
		errors.for_each([](const std::wstring& s) {
			std::wcout << L"Error: " << s << std::endl;
			});
		return { };
	}

	// Vrati se konfigurace.
	return configuration;
}

void ExecutionWindow::SetMetricId(scgms::SPersistent_Filter_Chain_Configuration& configuration, const GUID& metric_id)
{
	// Hleda se pozadovana metrika.
	configuration.for_each([&](scgms::SFilter_Configuration_Link link) {
		GUID id = Invalid_GUID;

		// Nepodarilo se najit.
		if (!Succeeded(link->Get_Filter_Id(&id))) {
			return;
		}

		// id se rovna chybovemu filtru.
		if (id == Error_Filter) {
			auto r = link.Resolve_Parameter(rsSelected_Metric);
			if (r) {
				r.set_GUID(metric_id);
			}
		}
		});
}

void ExecutionWindow::MetricThread()
{
	refcnt::Swstr_list errors;
	HRESULT rc{};
	ThreadObject* currObject;

	// pomocna funkce - vyblejeme na konzoli vsechny chyby, pokud nejake jsou
	auto printErrorsIfAny = [errors]() {
		errors.for_each([](auto str) {
			std::wcerr << str << std::endl;
			});
	};

	// Smycka pro provadeni vypoctu metrik.
	while (!metricQueue.empty() || !metricsDone) {
		// Pokud se ve fronte neco nachazi.
		if (!metricQueue.empty()) {
			// Vybere se hodnota z fronty.
			currObject = metricQueue.pop();

			// Pro jistotu se ulozi konfigurace.
			scgms::SPersistent_Filter_Chain_Configuration curr_conf = currObject->configuration;

			// Vypocita se hodnota metriky.
			double currentMetric = CalculateCurrentMetric(curr_conf);

			// Hodnota se ulozi do fronty vysledku.
			resultsQueue.push(result{ currObject->index, currentMetric, curr_conf });

			// Progressbaru s eposle dignal, ze se dopocitala metriky.
			this->ShowProgress(COUNT_PART_DONE);
		}
		// Ve fronte se zaim nic nenachazi, vlakno se uspi.
		else {
			std::this_thread::sleep_for(std::chrono::milliseconds(THREAD_SLEEP));
		}
	}
}

void ExecutionWindow::OptimizeThread()
{
	refcnt::Swstr_list errors;
	HRESULT rc{};
	ThreadObject* currObject;
	BlockingQueue<result> tempResultsQueue;

	// Pomocna funkce - vypise na konzoli vsechny chyby, pokud nejake jsou.
	auto printErrorsIfAny = [errors]() {
		errors.for_each([](auto str) {
			std::wcerr << str << std::endl;
			});
	};

	// Smycka pro optimalizaci parametru.
	while (running) {
		// Pokud fronta neni prazdna a matriy se uz dopocitaly.
		if (!optimizeQueue.empty() && metricsDone) {
			
			if (sortNeeded) {
				sortNeeded = false;
				// Je potreba vyndat vysledky z fronty a dat je do vektoru.
				while (!resultsQueue.empty()) {
					resultsVector.push_back(resultsQueue.pop());
				}

				// Vysledky mohou byt prehazene, nez jsou potreba, proto je potreba je seradit.
				std::sort(resultsVector.begin(), resultsVector.end(),
					// Funkce pro porovnani struktury result.
					[](const result& r1, const result& r2) {
						return (r1.index < r2.index);
					});

				foreach(result currResult, resultsVector) {
					tempResultsQueue.push(currResult);
				}
			}

			// Vybere se hodnota z fronty.
			currObject = optimizeQueue.pop();

			// Pro jistotu se ulozi konfigurace.
			scgms::SPersistent_Filter_Chain_Configuration curr_conf = tempResultsQueue.pop().configuration;

			// Provede se optimalizace parametru.
			bool success = Optimize(curr_conf, 5, L"Model_Bounds");
			// Optimalizace se nepovedla.
			if (!success) {
				return;
			}

			// Ulozi se optimalizovana konfigurace i se scenarem.
			success = SaveOptimizedConfiguration(curr_conf, OUTPUT_DIR, currObject->configFileName, currObject->metricDescriptor->id);
			// Ulozeni se nepovedlo.
			if (!success) {
				return;
			}

			// Vypocte se nova hodnota metriky.
			double currentMetric = CalculateCurrentMetric(curr_conf);

			// Hodnota se prida do fronty vysledku.
			resultsVector.push_back(result{ currObject->index, currentMetric });
		}
		// Fronta je prazdna nebo probiha vypocet v ostatnich vlaknech.
		else {
			std::this_thread::sleep_for(std::chrono::milliseconds(THREAD_SLEEP));
		}
	}

	// Vypisi se vysledky do vystupniho souboru.
	PrintOutResults();
}

HRESULT IfaceCalling PromiseErrorMetric(scgms::IFilter* filter, const void* data)
{
	if (scgms::SSignal_Error_Inspection insp = scgms::SSignal_Error_Inspection{ filter }) {
		// Do data bude ulozena hodnota metriky.
		insp->Promise_Metric(scgms::All_Segments_Id, reinterpret_cast<double*>(const_cast<void*>(data)), TRUE);
	}

	return S_OK;
}

double ExecutionWindow::CalculateCurrentMetric(scgms::SPersistent_Filter_Chain_Configuration& configuration)
{
	refcnt::Swstr_list errors;

	double metricValue = std::numeric_limits<double>::quiet_NaN();

	// Spusteni konfigurace.
	scgms::SFilter_Executor executor(configuration, &PromiseErrorMetric, &metricValue, errors, nullptr);

	// Nepovedlo se - vypisi se vsechny chyby.
	if (!executor) {
		errors.for_each([](const std::wstring& s) {
			std::wcout << L"Error: " << s << std::endl;
			});

		// Vrati se NaN.
		return std::numeric_limits<double>::quiet_NaN();
	}

	// Nepevdlo se - vrati se NaN.
	if (!Succeeded(executor->Terminate(TRUE))) {
		return std::numeric_limits<double>::quiet_NaN();
	}

	// Vrati se hodnota metriky.
	return metricValue;
}

bool ExecutionWindow::Optimize(scgms::SPersistent_Filter_Chain_Configuration& configuration, size_t filterIndex, const std::wstring& parametersName)
{
	refcnt::Swstr_list errors;
	HRESULT rc = E_FAIL;
	solver::TSolver_Progress solv_progress = solver::Null_Solver_Progress;

	// Promenna pro udrzeni smycky.
	std::atomic<bool> optimizing_flag{ true };

	// Vytvori se vlakno, ktere bude provaet optimalizaci.
	std::thread optimizing_thread([&] {

		// Zavola metodu pro optimalizaci parametru.
		rc = scgms::Optimize_Parameters(
			configuration,
			filterIndex, parametersName.c_str(), /* Co se optimalizuje. */
			nullptr, nullptr,
			solvers[chosen_solver].id,
			population, generation, // rychlejsi - 10, 20 pomalejsi - 20, 100
			nullptr, 0, 
			solv_progress, /* Progres prubehu. */
			errors);

		// Pripadne chyby, ktere se vypisi.
		errors.for_each([](const std::wstring& s) {
			std::wcout << L"Error: " << s << std::endl;
			});

		// Promenna pro udrzeni smycky.
		optimizing_flag = false;
		});

	double recent_percentage = std::numeric_limits<double>::quiet_NaN();
	double recent_fitness = std::numeric_limits<double>::quiet_NaN();

	// Label vypise, ze se optimalizuje.
	statusLabel->setText("Optimizing");

	// Smycka, ktera pokracuje, dokud nebude hotovy vypocet, postup je posilan do progressbaru.
	while (optimizing_flag) {
		if (solv_progress.max_progress != 0) {
			double current_percentage = static_cast<double>(solv_progress.current_progress) / static_cast<double>(solv_progress.max_progress);
			current_percentage = std::trunc(current_percentage * 1000.0);
			current_percentage *= 0.1;
			current_percentage = std::min(current_percentage, 100.0);

			if (recent_percentage != current_percentage) {
				recent_percentage = current_percentage;

				// Progressbaru se preda progres procesu.
				this->ShowProgress(current_percentage);

				const double tmp_best = solv_progress.best_metric[0];
				if ((recent_fitness != tmp_best) && (!std::isnan(tmp_best))) {
					recent_fitness = tmp_best;
				}
			}
		}

		// Vlakno se uspi.
		std::this_thread::sleep_for(std::chrono::milliseconds(THREAD_SLEEP));
	}

	// Posle se signal, ze byla vypoctena metrika.
	this->ShowProgress(COUNT_PART_DONE);

	// Optimlaizacni vlakno se da do join stavu.
	if (optimizing_thread.joinable()) {
		optimizing_thread.join();
	}

	// Nastala chyba.
	if (!Succeeded(rc)) {
		return false;
	}


	return true;
}

bool ExecutionWindow::SaveOptimizedConfiguration(scgms::SPersistent_Filter_Chain_Configuration& configuration, const std::wstring& outputDir, const std::wstring& config_file, GUID metricId)
{
	// Nazev slozky, do ktere budou soubory nakopirovany - nazev konfiguracniho souboru a GUID metriky.
	std::wstring dirName = config_file + L"_" + GUID_To_WString(metricId);

	// Cesta ke slozce.
	const filesystem::path Out_Base = Get_Application_Dir() / outputDir / dirName;

	// Pokud slozka neexistuje, vytvori se.
	if (!filesystem::exists(Out_Base) || !filesystem::is_directory(Out_Base)) {
		filesystem::create_directories(Out_Base);
	}

	// Cesta k ulozenemu souboru.
	const std::wstring Out_Path = Out_Base / config_file;

	refcnt::Swstr_list errors;

	// Ulozeni konfigurace do souboru.
	auto rc = configuration->Save_To_File(Out_Path.c_str(), errors.get());
	// Nepovedlo se ulozeni.
	if (!Succeeded(rc))
	{
		// Vypisi se chyby.
		errors.for_each([](const std::wstring& s) {
			std::wcout << L"Error: " << s << std::endl;
			});

		return false;
	}

	// Pokud jiz soubor neexistuje, nakopiruje se.
	if (!filesystem::exists(Out_Base / QStringToStdWString(scenarioFileEdit->text()))) {
		filesystem::copy(Get_Application_Dir() / QStringToStdWString(configsDir->text()) / QStringToStdWString(scenarioFileEdit->text()), Out_Base / QStringToStdWString(scenarioFileEdit->text()));
	}

	return true;
}

void ExecutionWindow::ShowProgress(int percent)
{
	QMetaObject::invokeMethod(this, "InvokeShowProgress", Qt::QueuedConnection, Q_ARG(int, percent));
}

void ExecutionWindow::OpenFile(std::wstring fileName)
{
	outputFileStream.open(Get_Application_Dir() / fileName);
}

void ExecutionWindow::CloseFile()
{
	outputFileStream.close();
}

void ExecutionWindow::PrintOutResults()
{
	// Otevre soubor, kde se vypisi vysledky.
	OpenFile(QStringToStdWString(outputFile->text()));

	// Vysledky mohou byt prehazene, nez jsou potreba, proto je potreba je seradit.
	std::sort(resultsVector.begin(), resultsVector.end(),
		// Funkce pro porovnani struktury result.
		[](const result& r1, const result& r2) {
			return (r1.index < r2.index);
		});

	// Vypis nazvu sloupecku.
	outputFileStream << "metric_description";
	for (int i = 0; i < configsDelivered; ++i) {
		outputFileStream << ", metric_value, optimized_metric_value";
	}
	outputFileStream << "\n";

	int i = 0;

	// Jsou vybrany vsechny metriky.
	if (chooseAll == true) {
		foreach(scgms::TMetric_Descriptor curr_metric, metrics) {

			// Nazev metriky.
			outputFileStream << curr_metric.description;

			// Vsechny sloupecky daneho radku.
			for (int j = 0; j < 2 * configsDelivered; ++j) {
				outputFileStream << ", " << resultsVector[i * 2 * configsDelivered + j].value << ", " << resultsVector[i * 2 * configsDelivered + ++j].value;
			}
			outputFileStream << "\n";
			++i;
		}
	}
	else {

		// Nazev metriky.
		outputFileStream << metrics[chosenIndex].description << ", ";

		// Vsechny sloupecky daneho radku.
		for (int j = 0; j < configsDelivered * 2; j += 2) {
			outputFileStream << resultsVector[j].value << ", " << resultsVector[j + 1].value << ", ";
		}
		outputFileStream << "\n";
	}

	resultsVector.clear();

	CloseFile();
}
