#pragma once

#include "../../def.h"
#include "../../ThreadObject/ThreadObject.h"
#include "../../BlockingQueue/BlockingQueue.h"

#pragma warning( push )

typedef struct _result {
	int index;														// index vysledku pro nasledny format vystupu 
	double value;													// hodnota vysledku
	scgms::SPersistent_Filter_Chain_Configuration configuration;	// Konfigurace pro optimalizaci.
}result;

/**
* Trida aplikace pro spusteni vypoctu metrik..
*/
class ExecutionWindow: public QMainWindow, public virtual refcnt::CNotReferenced
{
	Q_OBJECT

public:

	/* Vytvori nove okno. */
	ExecutionWindow();

	virtual ~ExecutionWindow();

	/* Metoda pro zobrazeni soucasneho postupu. */
	Q_INVOKABLE void InvokeShowProgress(int percentage);

	/* Nacte konfiguraci ze souboru. */
	scgms::SPersistent_Filter_Chain_Configuration LoadConfiguration(const std::wstring& configFile);

	/* Ulozi oprimalizovanou konfiguraci do vystupni slozky. */
	bool SaveOptimizedConfiguration(scgms::SPersistent_Filter_Chain_Configuration& configuration, const std::wstring& outputDir, const std::wstring& configFile, GUID metricId);

	/* Nastavi id metriky. */
	void SetMetricId(scgms::SPersistent_Filter_Chain_Configuration& configuration, const GUID& metricId);

	/* Nastavuje progres QProgressBaru. */
	void ShowProgress(int percent);

	/* Optimalizuje konfiguraci nad zadanym filtrem. */
	bool Optimize(scgms::SPersistent_Filter_Chain_Configuration& configuration, size_t filterIndex, const std::wstring& parametersName);

	/* Prehraje konfiguraci a ziska aktualni hodnotu metriky. */
	double CalculateCurrentMetric(scgms::SPersistent_Filter_Chain_Configuration& configuration);

	/* Naplni fronty configuracemi metrik nad pacienty. */
	void HandleConfig(int index, const std::wstring& fileName);

	/* Obsluhuje vlakno pro vypocet metriky. */
	void MetricThread();

	/* Obsluhuje vlakno pro optimalizaci a vypocet metriky. */
	void OptimizeThread();

	/* Otevre soubor pro nsledn vstup hodnot metrik. */
	void OpenFile(std::wstring fileName);

	/* Uzavre soubor ptro vstup. */
	void CloseFile();

	/* Vypise vysledek hodnot metrik. */
	void PrintOutResults();

private slots:

	/* Funkce, ktera probehne pri stisknuti tlacitka START. */
	void OnStartButtonClick();

public:

	/* Hlavni widget. */
	QWidget* centralWidget;

	/* Hlavni layout. */
	QGridLayout* centralLayout;

	/* Progressbar ukazujici progres vypoctu. */
	QProgressBar* progress;

	/* Label pro vypis statusu behu. */
	QLabel* statusLabel;

	/* Label pro vlozeni nazvu slozky. */
	QLabel* configsDirLabel;

	/* Pole pro vlozeni nazvu slozky, ze ktere se budou nacitat soubory (konfigurace). */
	QLineEdit* configsDir;

	/* Label pro vlozeni nazvu scenare. */
	QLabel* scenarioFileLabel;

	/* Pole pro vyplneni nazvu souboru. */
	QLineEdit* scenarioFileEdit;

	/* Label pro nazev vystupniho souboru. */
	QLabel* outputFileLabel;

	/* Pole pro vlozeni nazvu vystupniho souboru. */
	QLineEdit* outputFile;

	/* Label pro oznaceni metriky. */
	QLabel* metricsLabel;

	/* ComboBox, obsahujici vsechny metriky. */
	QComboBox* metricsBox;

	/* Label pro vyber vsech metrik. */
	QLabel* checkAll;

	/* CheckBox pro vyber vsech metrik. */
	QCheckBox* checkAllBox;

	/* Label pro vyber poctu souboru. */
	QLabel* firstXLabel;

	/* SpinBox pro vyber poctu nahranych souboru. */
	QSpinBox* firstXBox;

	/* Label pro vyber solveru. */
	QLabel* solversLabel;

	/* ComboBox, obsahujici vsechny solvery. */
	QComboBox* solversBox;

	/* Label pro ziskani velikosti populace. */
	QLabel* populationLabel;

	/* SpinBox pro ziskani velikosti populace. */
	QSpinBox* populationBox;

	/* Label pro ziskani poctu generaci. */
	QLabel* generationLabel;

	/* SpinBox pro ziskani poctu generaci. */
	QSpinBox* generationBox;

	/* Tlacitko pro spusteni vypoctu. */
	QPushButton* startButton;

	/* Vektor vlaken pro vypocet metrik. */
	std::vector<std::unique_ptr<std::thread>> mSmartCGMSExecutionThreads;

	/* Vektor nactenych souboru. */
	std::vector<std::wstring> loadedConfigs;

	/* Fronta pro zpracovani vypoctu metrik. */
	BlockingQueue<ThreadObject*> metricQueue;

	/* Fronta pro zpracovani optimalizaci. */
	BlockingQueue<ThreadObject*> optimizeQueue;

	/* Vektor metrik ziskanych ze SmartCGMS. */
	std::vector<scgms::TMetric_Descriptor> metrics;

	/* Vektor solveru ziskanych ze SmartCGMS. */
	std::vector<scgms::TSolver_Descriptor> solvers;

	/* Solver pro optimize_parametres. */
	int chosen_solver = 0;

	/* Podminkova promenna pro udrzeni stavu, ze je dopocitano. */
	bool sortNeeded = true;

	/* Vyber vsech metrik. */
	bool chooseAll = false;

	/* Indexu vybrane metriky. */
	int chosenIndex = 0;

	/* Oznacuje, ze aplikace bezi. */
	bool running = true;

	/* Pocet nactenych konfiguraci. */
	int configsDelivered;

	/* Velikost populace. */
	int population = 20;

	/* Pocet generaci. */
	int generation = 100;

	/* Fronta pro uchovavani vysledku vypoctu. */
	BlockingQueue<result> resultsQueue;

	/* Vektor vyslednch hodnot pro nsledn vpis. */
	std::vector<result> resultsVector;

	/* Vystup do souboru pro vysledky. */
	std::wofstream outputFileStream;

	/* Pocet vybranych souboru z adresare. */
	int firstXValue;

	/* Pocet dostupnych metrik. */
	size_t numOfMetrics;

	/* Procento hotove prace. */
	double donePercentage;

	/* Promenna pro ovladani vlaken. */
	std::atomic<bool> metricsDone = false;
};

#pragma warning( pop )
