#pragma once

class Day;

#include "../../BaseConfigs/BaseConfigs.h"
#include "../../Week/Week.h"
#include "../../Day/Day.h"
#include "../../Day/DayEventValues/DayEventValues.h"
#include "../../Day/DayEventWidgets/DayEventWidgets.h"

#include "../../def.h"
#include "src_frontend/Windows/ExecutionWindow/ExecutionWindow.h"

#pragma warning( push )
#pragma warning( disable : 4250 ) // C4250 - 'class1' : inherits 'class2::member' via dominance

/*
 * Hlavni okno aplikace. Zde se vola spusteni generovani dat.
 */
class MainDefWindow : public QMainWindow, public virtual refcnt::CNotReferenced
{
	Q_OBJECT

	public:
		/* Vytvori nove okno. */
		MainDefWindow();
		virtual ~MainDefWindow();

		/* Inicializuje hodnoty a nastavi nemenne widgety. */
		void initWindow();

		/* Vytvori scenar za pomoci SmartCGMS.  */
		void SmartCGMS_Execution_Thread_Func(const std::wstring& fileName);

		/* Nastavi vek pacienta. */
		void setAgeValue(int ageValue);

		/* Nastavi typ diabetu. */
		void setDiabetValue(bool diabetValue);

		/* Nastavi pocet dnu v tdnu. */
		void setNumOfDays(int numOfDays);

		/* Nastavi pocet definovanych tydnu. */
		void setNumOfWeeks(int numOfWeeks);

	private slots:
		/* Prepne den na predesly. */
		void OnPrevButtonClick();

		/* Prepne den na nasledujici. */
		void OnNextButtonClick();

		/* Spusti simulaci. */
		void OnStartButtonClick();

		/* Ukonci simulaci. */
		void OnSimulationEnded();

	signals:

		/* Notifikuje, ze simulace skoncila. */
		void SimulationEnded();

	private:

		/* Vyplni ramec dne potrebnymi widgety. */
		void fillLayout(Day* day);

		/* Urci nahodnost casu udalosti. Max. hodnota, ktera muze nastate bude (jitter_value - 1). */
		int randomize_time(int jitter_value);

		/* Korektne ukonci simulaci. */
		bool Send_Shut_Down(double time = std::numeric_limits<double>::quiet_NaN());

		/* Marker zacatku segmentu. */
		bool Send_Time_Segment_Marker(bool start, double time);

		/* Marker pro generatory signalu (aby se mohly synchronizovat). */
		bool Send_Level(double value, double time, const GUID& signalId);

	private:

		/* Vek pacienta. */
		int ageValue;

		/* Typ diabetu (I. (true) nebo II. typ (false)).  */
		bool diabetValue;

		/* Pocet dni v tydnu. (7 nebo 1) */
		int numOfDays;

		/* Pocet tydnu simulace. */
		int numOfWeeks;

		/* Momentalni den. */
		int currentDay;

		/* Momentalni tyden. */
		int currentWeek;

		/* Hodnota progressbaru, slouzi k orientaci v pokorceni definice. */
		int progressValue;

		/* Prirustek progressbaru. */
		int progressIncrease;
		
		/* Vektor widgetu jedne akce dne. */
		std::vector<DayEventWidgets*> eventsWidgets;

		/* Widget uchovavajici vsechny dny. */
		QStackedWidget* stackedWidget;

		/* Progressbar ukazujici progres vypoctu. */
		QProgressBar* progress;

		/* Tlacitko pro spusteni simulace. */
		QPushButton* startButton;

		/* Label pro vypis statusu simulace. */
		QLabel* statusLabel;

		/* Pole pro vyplneni nazvu souboru. */
		QLineEdit* fileNAmeEdit;

		/* Vektor tydnu (naplni se podle poctu tydnu). */
		std::vector<Week *> weeks;

		/* Hlavni widget. */
		QWidget* centralWidget;

		/* Hlavni layout. */
		QGridLayout* centralLayout;

		/* Tlacitko pro prepnuti na predchozi den. */
		QPushButton* prevButton;

		/* Tlacitko pro prepnuti na dalsi den. */
		QPushButton* nextButton;

		/* Random generator. */
		std::random_device dev;

		/* SmartCGMS core execution instance */
		scgms::SFilter_Executor filterExecutor;

		/* Vektor vlaken pro vypocet metrik. */
		std::unique_ptr<std::thread> smartCGMSExecutionThread;

		/* Podminkova promenna pro notifikaci. */
		std::condition_variable readyCv;

		/* Mutex pro cekani na simulaci. */
		std::mutex readyMtx;

		/* Promenna pro mutex. */
		int ready = 0;
};

#pragma warning( pop )
