
#include "frechet_distance.h"

#include <cmath>

Frechet_Distance::Frechet_Distance(const scgms::TMetric_Parameters& params) : mParameters{ params } {

}

Frechet_Distance::~Frechet_Distance() {
	
	// Vycisti se vsechny vektory.
	Reference.clear();
	Calculated.clear();
	Times.clear();
	Maxims.clear();
}

double Frechet_Distance::Euclidean_Norm(double reference, double calculated, double time_i, double time_j)
{
	double dist, diffY, diffX;
	// Rozdil osy X.
	diffX = time_j - time_i;
	// Rozdil osy Y.
	diffY = reference - calculated;

	// Odmocnina ze souctu mocnin.
	dist = sqrt(diffX * diffX + diffY * diffY);

	return dist;
}

void Frechet_Distance::FindMaxims()
{
	double temp_max, current;



	// Prvni hodnoty se pocitaji zvlast.
	Maxims.push_back(Euclidean_Norm(Reference[0], Calculated[0], Times[0], Times[0]));

	// Permutace vsech  hodnot, z nich se vyberou ty nejvetsi.
	for (int i = 0; i < Reference.size(); ++i) {
		temp_max = 0.0;
		for (int j = i; j < Calculated.size(); ++j) {
			// Vypocet vzdalenosti mezi dvema body.
			current = Euclidean_Norm(Reference[i], Calculated[j], Times[i], Times[j]);
			if (current > temp_max) {
				temp_max = current;
			}
		}
		// Nejvetsi hodnota se prida do vektoru.
		Maxims.push_back(temp_max);
	}

	// Vycisti se vektor predanych hodnot aby mohl pokracovat pripadny vypocet.
	Reference.clear();
	Calculated.clear();
	Times.clear();
}

HRESULT IfaceCalling Frechet_Distance::Accumulate(const double* times, const double* reference, const double* calculated, const size_t count) 
{
	// Predaji se vsechny ziskane hodnoty.
	for (int i = 0; i < count; ++i) {
		Reference.push_back(reference[i]);
		Calculated.push_back(calculated[i]);
		Times.push_back(times[i]);
	}

	return S_OK;
}

HRESULT IfaceCalling Frechet_Distance::Reset()
{
	// Vsechny vektory se vycisti.
	Maxims.clear();
	Reference.clear();
	Calculated.clear();
	Times.clear();

	return S_OK;
}

HRESULT IfaceCalling Frechet_Distance::Calculate(double* metric, size_t* levels_accumulated, size_t levels_required) {

	// Ziskal se pozadovany pocet hdnot?
	*levels_accumulated = Reference.size();
	if (*levels_accumulated < std::max(static_cast<size_t>(1), levels_required)) {
		return S_FALSE;
	}

	// Najdou se vsechna maxima.
	FindMaxims();

	// Z maxim se vybere minimum.
	for (int i = 0; i < *levels_accumulated; ++i) {
		if (Maxims[i] < metricValue) {
			metricValue = Maxims[i];
		}
	}
	
	// Preda se vysledna hodnota.
	*metric = metricValue;

	return S_OK;
}

HRESULT IfaceCalling Frechet_Distance::Get_Parameters(scgms::TMetric_Parameters* parameters) {

	// Zkopiruji se parametry.
	memcpy(parameters, &mParameters, sizeof(mParameters));

	return S_OK;
}
