
#pragma once

#include <rtl/FilterLib.h>
#include <rtl/referencedImpl.h>
#include <rtl/SolverLib.h>
#include <rtl/UILib.h>

#include <limits>

#pragma warning( push )
#pragma warning( disable : 4250 ) // C4250 - 'class1' : inherits 'class2::member' via dominance

/* Definice tridy implementujici Frechetovu vzdalenost. */
class Frechet_Distance : public virtual scgms::IMetric, public virtual refcnt::CReferenced {

protected:
	/* Parametr pro SmartCMGS */
	const scgms::TMetric_Parameters mParameters;
	
	/* Vektor refenrcnich hodnot. */
	std::vector<double> Reference;
	
	/* Vektor vypocitanych hodnot. */
	std::vector<double> Calculated;

	std::vector<double> Times;

	/* Vektor maxim dane iterace. */
	std::vector<double> Maxims;


public:
	/* Vytvori novou instanci objektu. */
	Frechet_Distance(const scgms::TMetric_Parameters& params);
	virtual ~Frechet_Distance();

	/* Vypocte euklidovskou vdalenost mezi dvema body. */
	double Euclidean_Norm(double reference, double calculated, double time_i, double time_j);

	/* Nalezne maxima pro kazdou iteraci. */
	void FindMaxims();

	/* Preradi zsikane hodnoty. */
	virtual HRESULT IfaceCalling Accumulate(const double* times, const double* reference, const double* calculated, const size_t count) final;

	/* Vycisti vektory hodnot. */
	virtual HRESULT IfaceCalling Reset() final;

	/* Vypocita hodnotu metriky. */
	virtual HRESULT IfaceCalling Calculate(double* metric, size_t* levels_accumulated, size_t levels_required) final;

	/* Vrati parametr. */
	virtual HRESULT IfaceCalling Get_Parameters(scgms::TMetric_Parameters* parameters) final;

private:

	/* Hodnota nalezeneho minima (hodnota metriky). */
	double metricValue = DBL_MAX;
};

#pragma warning( pop )
